//------------------------------
class Flake{
  float x, y, z;
  float rx, ry, rz;
  float randX, randY, randZ;

  String[] types = {
    "kick", "snare", "hat","snow","snow","snow"  };
  String type = "";
  float dia;

  PImage imageX;
  PImage imageY;
  PImage imageZ;

  color snow;
  color kickCol;
  color snareCol;
  color hatCol;

  float kickFlakeSize;
  float snareFlakeSize;
  float hatFlakeSize;
  float snowSize;
  
  float gravity;
  float noiseAmp;

  Flake(PImage imageX, PImage imageY, PImage imageZ, 
  color snow, color kickCol, color snareCol, color hatCol, 
  float kickFlakeSize, float snareFlakeSize, float hatFlakeSize, float snowSize,
  float gravity, float noiseAmp){
    this.imageX = imageX;
    this.imageY = imageY;
    this.imageZ = imageZ;

    this.snow = snow;
    this.kickCol = kickCol;
    this.snareCol = snareCol;
    this.hatCol = hatCol;
    this.kickFlakeSize = kickFlakeSize;
    this.snareFlakeSize = snareFlakeSize;
    this.hatFlakeSize = hatFlakeSize;
    this.snowSize = snowSize;
    
    this.gravity = gravity;
    this.noiseAmp = noiseAmp;

    x = random(0, res);
    y = random(0, res); 
    z = random(0, res);
    rx = random(0, TWO_PI);
    ry = random(0, TWO_PI);
    rz = random(0, TWO_PI);    

    type = types[int(random(0,6))];
  }

  void render(){
    // define new random postion:
    x = x + randX;
    y = y + randY - gravity;
    z = z + randZ;

    // do flake rotation:
    rx = rx + TWO_PI/120;
    ry = ry + TWO_PI/120; 
    rz = rz + TWO_PI/120;

    pushMatrix();
      translate(x, y, z);      
      rotateX(rx);
      rotateY(ry);
      rotateZ(rz);
    
      // Render the flake mesh:
      flakeMesh();
    popMatrix();

    // if out of bounds, reset:
    if(y < 0){
      y=res;
    }
    else if(y>res){
      y = 0; 
    }
    if(x < 0){
      x = res;
    }   
    else if(x>res){
      x=0; 
    }
    if(z < 0){
      z=res;
    }  
    else if(z>res){
      z=0;
    }    
  }

  void rand(){
    float mx = map(x, 0, res, 0, winRes);
    float my = map(y, 0, res, 0, winRes);
    float mz = map(z, 0, res, 0, winRes);

    color xCol = imageX.get(int(mz), int(my));
    color yCol = imageY.get(int(mx), int(mz));
    color zCol = imageZ.get(int(mx), int(my)); 

    float xVal = map(brightness(xCol), 0.0, 255.0, -1.0, 1.0);
    float yVal = map(brightness(yCol), 0.0, 255.0, -1.0, 1.0);
    float zVal = map(brightness(zCol), 0.0, 255.0, -1.0, 1.0);  

    // scale up the speed if they're on the beat:
    float beatScale=1;
    if(type == "kick"){
      beatScale = kickSize;;
    }
    if(type == "snare"){
      beatScale = snareSize;    
    }    
    if(type == "hat"){
      beatScale = hatSize; 
    }      

    randX = (xVal *  noiseAmp*beatScale);
    randY = (yVal *  noiseAmp*beatScale);
    randZ = (zVal *  noiseAmp*beatScale);
  }

  void flakeMesh(){
    // make flakes fade to black as they opproach the boundary:
    float dFromC = dist(x,y,z, center, center,center)-center/2;
    float dClamp = constrain(dFromC, 0, center);
    float dMap = map(dClamp, 0, center, 0, 1);
    color black = color(0,0,0);
    
    stroke(lerpColor(snow, black, dMap));
    strokeWeight(1);
    dia = snowSize;
    
    if(type == "kick"){
      dia = kickFlakeSize*kickSize;
      color newCol = lerpColor(kickCol, color(0,0,0), dMap);
      stroke(newCol);
    }
    if(type == "snare"){
      dia = snareFlakeSize*snareSize;
      stroke(lerpColor(snareCol, color(0,0,0), dMap));
    }    
    if(type == "hat"){
      dia = hatFlakeSize*hatSize; 
      stroke(lerpColor(hatCol, color(0,0,0), dMap));
    }   

    float[] xes = {
      0, .75, 1, .75, 0, -.75, -1, -.75     };       
    float[] yes = {
      -1, -.75, 0, .75, 1, .75, 0, -.75     };

    pushMatrix();  
    scale(dia); 

    beginShape(LINES);
    for(int m=0; m<xes.length; m++){
      vertex(0, 0, 0);     
      vertex(xes[m], yes[m], 0);
    }
    endShape();
    popMatrix();
  }

  void run(){
    rand();
    render();
  }

}









