import processing.core.*; 
import processing.xml.*; 

import ddf.minim.*; 
import ddf.minim.analysis.*; 

import java.applet.*; 
import java.awt.*; 
import java.awt.image.*; 
import java.awt.event.*; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class GrooveFlakes01 extends PApplet {

/*
GrooveFlakes01
 Eric Pavey - 2008-12-28  
 warpcat@sbcglobal.net
 warpcat.blogspot.com
 
 Render a 3d scene with falling snowflakes.
 The flakes fall 'through' perlin noise, modulating
 their position.
 Soundtrack by GrooveSalad - somaFM.com
 */

//------------------------------
// Import in the Minim audio lib:
// Download here: http://code.compartmental.net/tools/minim/



//------------------------------
// User vars:

// What height to make the window?  Width is auto-set to
// be 1.5 that amount.
int winRes = 512;
// Scale the whole scene.  Mainly just effects the relative
// size of the flakes:
int terrainSize = 2;


// How much turbulance to the noise?
float noiseFreq = random(2,6);
// How much amplitude to the noise?
float noiseAmp = random(3,6);
// Gravity should be at least half the noiseAmp, if you
// want to keep the flakes falling down
float gravity =noiseAmp/2;

// How many total snowflakes?
int numFlakes = 3000;

// Audio related vars:
// How big should our differnt kind of flakes be?
float kickFlakeSize = 2.0f;
float snareFlakeSize = 2.0f;
float hatFlakeSize = 2.0f;
float snowSize = 4.0f;
// how much bigger should they scale when on the beat?
float beatScale = 4.0f;

// Define colors:
int sky = color(0,0,0);
int snow = color(255,255,255);

int kickCol = color(255,0,0);
int snareCol = color(0,255,0);
int hatCol = color(255,255,0);

//------------------------------
// System vars, no touch!
int depth = winRes;
Flake[] flakes = new Flake[numFlakes];
PImage imageX; 
PImage imageY; 
PImage imageZ;
int res = winRes * terrainSize;
float center = res / 2;
float[] cEye = {
  center, center, center};

// Audio stuff:
Minim minim;
AudioPlayer song;
BeatDetect beat;
BeatListener bl;
float kickSize, snareSize, hatSize;

// Text stuff:
PFont fontA = createFont("Arial", 32, true);    

//------------------------------
public void setup(){
  //size(int(winRes*1.5),winRes, P3D);
  size(768,512, P3D);
  frameRate(30);
  smooth(); 

  imageX = createImage(winRes, winRes, ARGB);
  imageY = createImage(winRes, winRes, ARGB);
  imageZ = createImage(winRes, winRes, ARGB);  

  makePerlin(imageX, noiseFreq);
  makePerlin(imageY, noiseFreq);
  makePerlin(imageZ, noiseFreq);  

  for(int i=0; i<numFlakes; i++){
    flakes[i] = new Flake(imageX, imageY, imageZ, 
    snow, kickCol, snareCol, hatCol, 
    kickFlakeSize, snareFlakeSize, hatFlakeSize, snowSize,
    gravity, noiseAmp); 

  }

  // Start our audio playing:
  minim = new Minim(this);
  // Load up the GrooveSalad playlist (from www.somafm.com), 
  // and give the AudioPlayer buffers that are 2048 samples long
  //song = minim.loadFile("http://160.79.128.242:8032", 2048);
  song = minim.loadFile("http://streamer-ntc-aa08.somafm.com:80/stream/1018", 2048);  
  // play the file
  song.play();  
  // Setup our beat detection:
  beat = new BeatDetect(song.bufferSize(), song.sampleRate());
  // sensitivty in milliseconds
  beat.setSensitivity(10);  
  bl = new BeatListener(beat, song); 

  // Bug:  If for some reason we don't render a box
  // a weird rendering anomoly shows up:
  // It's like certain flakes render in camera-space...?  
  box(1, 1, 1);
}

//------------------------------
public void draw(){
  // Audio detection:
  if ( beat.isKick() ) kickSize = kickFlakeSize*beatScale;
  if ( beat.isSnare() ) snareSize = snareFlakeSize*beatScale;
  if ( beat.isHat() ) hatSize = hatFlakeSize*beatScale;

  background(sky);
  // Flip the corrdinate system, so +y goes up (like Maya)
  // This helps my brain.
  rotateX(PI);
  translate(0, -height, 0);

  // setup the perspective camera lense (from Processing examples)
  float fov = PI/2.0f;
  float cameraZ = (height/2.0f) / tan(PI * fov / 360.0f);
  perspective(fov, PApplet.parseFloat(width)/PApplet.parseFloat(height), cameraZ/10.0f, cameraZ*10.0f);

  // define our camera look at point:
  float rot = map(mouseX, 0, width, 0, TWO_PI);
  float pX = cEye[0] + cos(rot)*winRes;
  float pZ = cEye[2] + sin(rot)*winRes;  
  float pY = map(mouseY, 0, PApplet.parseFloat(height), 0, PApplet.parseFloat(height*2));

  camera(cEye[0], cEye[1], cEye[2], pX, pY, pZ, 0, -1, 0);

  // draw the center axis (debug)
/*
  stroke(255,0,0);
  strokeWeight(1);
  line(0,0,0, res,0,0);
  stroke(0,255,0);
  line(0,0,0, 0,res,0); 
  stroke(0,0,255);
  line(0,0,0, 0,0,res);  
  */


  // draw our background:
  stroke(128,128,128);
  line(0,0,0, res,0,0);
  line(0,0,0, 0,0,res);    
  line(res,0,res, res,0,0);
  line(res,0,res, 0,0,res); 

  line(0,res,0, res,res,0);
  line(0,res,0, 0,res,res);    
  line(res,res,res, res,res,0);
  line(res,res,res, 0,res,res);   


  // Render our showflakes
  for(int f=0; f<numFlakes; f++){
    flakes[f].run();  
  }

  displayText();
  // Now bring the beat size back down:
  kickSize = constrain(kickSize * 0.9f, 1, beatScale);
  snareSize = constrain(snareSize * 0.9f, 1, beatScale);
  hatSize = constrain(hatSize * 0.9f, 1, beatScale);
}

//------------------------------
// Stop Minim when over:
public void stop()
{
  // always close Minim audio classes when you are done with them
  song.close();
  minim.stop();
  super.stop();
}


//------------------------------
public void displayText(){
  fill(255,128);
  String lineA = "GrooveFlakes";
  String lineB = "Powered by GrooveSalad";
  String lineC = "somaFM.com";
  textFont(fontA);
  
  pushMatrix();
  translate(center, center, center*1.5f);
  rotateX(-PI);
  text(lineA,0,0,0);
  translate(0,20,0);
  scale(.50f,.50f,.50f);
  text(lineB,0,0,0);
  translate(0,30,0);
  text(lineC,0,0,0);
  popMatrix();

}

public void keyPressed(){
  if(key == 'p'){  
    saveFrame("GrooveFlakes-####.jpg");
    println("Screengrab!");
  }
}


// from the example file "Minim->BeatDetect->FrequencyEnergy"
class BeatListener implements AudioListener
{
  private BeatDetect beat;
  private AudioPlayer source;
  
  BeatListener(BeatDetect beat, AudioPlayer source)
  {
    this.source = source;
    this.source.addListener(this);
    this.beat = beat;
  }
  
  public void samples(float[] samps)
  {
    beat.detect(source.mix);
  }
  
  public void samples(float[] sampsL, float[] sampsR)
  {
    beat.detect(source.mix);
  }
}
//------------------------------
class Flake{
  float x, y, z;
  float rx, ry, rz;
  float randX, randY, randZ;

  String[] types = {
    "kick", "snare", "hat","snow","snow","snow"  };
  String type = "";
  float dia;

  PImage imageX;
  PImage imageY;
  PImage imageZ;

  int snow;
  int kickCol;
  int snareCol;
  int hatCol;

  float kickFlakeSize;
  float snareFlakeSize;
  float hatFlakeSize;
  float snowSize;
  
  float gravity;
  float noiseAmp;

  Flake(PImage imageX, PImage imageY, PImage imageZ, 
  int snow, int kickCol, int snareCol, int hatCol, 
  float kickFlakeSize, float snareFlakeSize, float hatFlakeSize, float snowSize,
  float gravity, float noiseAmp){
    this.imageX = imageX;
    this.imageY = imageY;
    this.imageZ = imageZ;

    this.snow = snow;
    this.kickCol = kickCol;
    this.snareCol = snareCol;
    this.hatCol = hatCol;
    this.kickFlakeSize = kickFlakeSize;
    this.snareFlakeSize = snareFlakeSize;
    this.hatFlakeSize = hatFlakeSize;
    this.snowSize = snowSize;
    
    this.gravity = gravity;
    this.noiseAmp = noiseAmp;

    x = random(0, res);
    y = random(0, res); 
    z = random(0, res);
    rx = random(0, TWO_PI);
    ry = random(0, TWO_PI);
    rz = random(0, TWO_PI);    

    type = types[PApplet.parseInt(random(0,6))];
  }

  public void render(){
    // define new random postion:
    x = x + randX;
    y = y + randY - gravity;
    z = z + randZ;

    // do flake rotation:
    rx = rx + TWO_PI/120;
    ry = ry + TWO_PI/120; 
    rz = rz + TWO_PI/120;

    pushMatrix();
      translate(x, y, z);      
      rotateX(rx);
      rotateY(ry);
      rotateZ(rz);
    
      // Render the flake mesh:
      flakeMesh();
    popMatrix();

    // if out of bounds, reset:
    if(y < 0){
      y=res;
    }
    else if(y>res){
      y = 0; 
    }
    if(x < 0){
      x = res;
    }   
    else if(x>res){
      x=0; 
    }
    if(z < 0){
      z=res;
    }  
    else if(z>res){
      z=0;
    }    
  }

  public void rand(){
    float mx = map(x, 0, res, 0, winRes);
    float my = map(y, 0, res, 0, winRes);
    float mz = map(z, 0, res, 0, winRes);

    int xCol = imageX.get(PApplet.parseInt(mz), PApplet.parseInt(my));
    int yCol = imageY.get(PApplet.parseInt(mx), PApplet.parseInt(mz));
    int zCol = imageZ.get(PApplet.parseInt(mx), PApplet.parseInt(my)); 

    float xVal = map(brightness(xCol), 0.0f, 255.0f, -1.0f, 1.0f);
    float yVal = map(brightness(yCol), 0.0f, 255.0f, -1.0f, 1.0f);
    float zVal = map(brightness(zCol), 0.0f, 255.0f, -1.0f, 1.0f);  

    // scale up the speed if they're on the beat:
    float beatScale=1;
    if(type == "kick"){
      beatScale = kickSize;;
    }
    if(type == "snare"){
      beatScale = snareSize;    
    }    
    if(type == "hat"){
      beatScale = hatSize; 
    }      

    randX = (xVal *  noiseAmp*beatScale);
    randY = (yVal *  noiseAmp*beatScale);
    randZ = (zVal *  noiseAmp*beatScale);
  }

  public void flakeMesh(){
    // make flakes fade to black as they opproach the boundary:
    float dFromC = dist(x,y,z, center, center,center)-center/2;
    float dClamp = constrain(dFromC, 0, center);
    float dMap = map(dClamp, 0, center, 0, 1);
    int black = color(0,0,0);
    
    stroke(lerpColor(snow, black, dMap));
    strokeWeight(1);
    dia = snowSize;
    
    if(type == "kick"){
      dia = kickFlakeSize*kickSize;
      int newCol = lerpColor(kickCol, color(0,0,0), dMap);
      stroke(newCol);
    }
    if(type == "snare"){
      dia = snareFlakeSize*snareSize;
      stroke(lerpColor(snareCol, color(0,0,0), dMap));
    }    
    if(type == "hat"){
      dia = hatFlakeSize*hatSize; 
      stroke(lerpColor(hatCol, color(0,0,0), dMap));
    }   

    float[] xes = {
      0, .75f, 1, .75f, 0, -.75f, -1, -.75f     };       
    float[] yes = {
      -1, -.75f, 0, .75f, 1, .75f, 0, -.75f     };

    pushMatrix();  
    scale(dia); 

    beginShape(LINES);
    for(int m=0; m<xes.length; m++){
      vertex(0, 0, 0);     
      vertex(xes[m], yes[m], 0);
    }
    endShape();
    popMatrix();
  }

  public void run(){
    rand();
    render();
  }

}









//-----------------------------
// Make a 'perlin noise image'.
// Fill the define image with noise.
// channel is 0-2, = r, g, or b

public void makePerlin(PImage img, float noiseScale){
  noiseSeed(PApplet.parseInt(random(0,1000)));
  
  for(int x=0; x < res; x++) {
    for(int y=0; y < res; y++){
      // normalize the x & y positions between 0-1
      float normX = map(x, 0, res, 0, noiseScale);
      float normY = map(y, 0, res, 0, noiseScale);  
      // find the noise value for that x,y pos:    
      float val = noise(normX, normY);
      // map that value to a color:
      int cVal = PApplet.parseInt(map(val, 0, 1, 0, 255));

      int col = color(cVal);     
      img.set(x,y,col);     
    }
  }  
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#ece9d8", "GrooveFlakes01" });
  }
}
