/*
GrooveFlakes01
 Eric Pavey - 2008-12-28  
 warpcat@sbcglobal.net
 warpcat.blogspot.com
 
 Render a 3d scene with falling snowflakes.
 The flakes fall 'through' perlin noise, modulating
 their position.
 Soundtrack by GrooveSalad - somaFM.com
 */

//------------------------------
// Import in the Minim audio lib:
// Download here: http://code.compartmental.net/tools/minim/
import ddf.minim.*;
import ddf.minim.analysis.*;

//------------------------------
// User vars:

// What height to make the window?  Width is auto-set to
// be 1.5 that amount.
int winRes = 512;
// Scale the whole scene.  Mainly just effects the relative
// size of the flakes:
int terrainSize = 2;


// How much turbulance to the noise?
float noiseFreq = random(2,6);
// How much amplitude to the noise?
float noiseAmp = random(3,6);
// Gravity should be at least half the noiseAmp, if you
// want to keep the flakes falling down
float gravity =noiseAmp/2;

// How many total snowflakes?
int numFlakes = 3000;

// Audio related vars:
// How big should our differnt kind of flakes be?
float kickFlakeSize = 2.0;
float snareFlakeSize = 2.0;
float hatFlakeSize = 2.0;
float snowSize = 4.0;
// how much bigger should they scale when on the beat?
float beatScale = 4.0;

// Define colors:
color sky = color(0,0,0);
color snow = color(255,255,255);

color kickCol = color(255,0,0);
color snareCol = color(0,255,0);
color hatCol = color(255,255,0);

//------------------------------
// System vars, no touch!
int depth = winRes;
Flake[] flakes = new Flake[numFlakes];
PImage imageX; 
PImage imageY; 
PImage imageZ;
int res = winRes * terrainSize;
float center = res / 2;
float[] cEye = {
  center, center, center};

// Audio stuff:
Minim minim;
AudioPlayer song;
BeatDetect beat;
BeatListener bl;
float kickSize, snareSize, hatSize;

// Text stuff:
PFont fontA = createFont("Arial", 32, true);    

//------------------------------
void setup(){
  //size(int(winRes*1.5),winRes, P3D);
  size(768,512, P3D);
  frameRate(30);
  smooth(); 

  imageX = createImage(winRes, winRes, ARGB);
  imageY = createImage(winRes, winRes, ARGB);
  imageZ = createImage(winRes, winRes, ARGB);  

  makePerlin(imageX, noiseFreq);
  makePerlin(imageY, noiseFreq);
  makePerlin(imageZ, noiseFreq);  

  for(int i=0; i<numFlakes; i++){
    flakes[i] = new Flake(imageX, imageY, imageZ, 
    snow, kickCol, snareCol, hatCol, 
    kickFlakeSize, snareFlakeSize, hatFlakeSize, snowSize,
    gravity, noiseAmp); 

  }

  // Start our audio playing:
  minim = new Minim(this);
  // Load up the GrooveSalad playlist (from www.somafm.com), 
  // and give the AudioPlayer buffers that are 2048 samples long
  //song = minim.loadFile("http://160.79.128.242:8032", 2048);
  song = minim.loadFile("http://streamer-ntc-aa08.somafm.com:80/stream/1018", 2048);  
  // play the file
  song.play();  
  // Setup our beat detection:
  beat = new BeatDetect(song.bufferSize(), song.sampleRate());
  // sensitivty in milliseconds
  beat.setSensitivity(10);  
  bl = new BeatListener(beat, song); 

  // Bug:  If for some reason we don't render a box
  // a weird rendering anomoly shows up:
  // It's like certain flakes render in camera-space...?  
  box(1, 1, 1);
}

//------------------------------
void draw(){
  // Audio detection:
  if ( beat.isKick() ) kickSize = kickFlakeSize*beatScale;
  if ( beat.isSnare() ) snareSize = snareFlakeSize*beatScale;
  if ( beat.isHat() ) hatSize = hatFlakeSize*beatScale;

  background(sky);
  // Flip the corrdinate system, so +y goes up (like Maya)
  // This helps my brain.
  rotateX(PI);
  translate(0, -height, 0);

  // setup the perspective camera lense (from Processing examples)
  float fov = PI/2.0;
  float cameraZ = (height/2.0) / tan(PI * fov / 360.0);
  perspective(fov, float(width)/float(height), cameraZ/10.0, cameraZ*10.0);

  // define our camera look at point:
  float rot = map(mouseX, 0, width, 0, TWO_PI);
  float pX = cEye[0] + cos(rot)*winRes;
  float pZ = cEye[2] + sin(rot)*winRes;  
  float pY = map(mouseY, 0, float(height), 0, float(height*2));

  camera(cEye[0], cEye[1], cEye[2], pX, pY, pZ, 0, -1, 0);

  // draw the center axis (debug)
/*
  stroke(255,0,0);
  strokeWeight(1);
  line(0,0,0, res,0,0);
  stroke(0,255,0);
  line(0,0,0, 0,res,0); 
  stroke(0,0,255);
  line(0,0,0, 0,0,res);  
  */


  // draw our background:
  stroke(128,128,128);
  line(0,0,0, res,0,0);
  line(0,0,0, 0,0,res);    
  line(res,0,res, res,0,0);
  line(res,0,res, 0,0,res); 

  line(0,res,0, res,res,0);
  line(0,res,0, 0,res,res);    
  line(res,res,res, res,res,0);
  line(res,res,res, 0,res,res);   


  // Render our showflakes
  for(int f=0; f<numFlakes; f++){
    flakes[f].run();  
  }

  displayText();
  // Now bring the beat size back down:
  kickSize = constrain(kickSize * 0.9, 1, beatScale);
  snareSize = constrain(snareSize * 0.9, 1, beatScale);
  hatSize = constrain(hatSize * 0.9, 1, beatScale);
}

//------------------------------
// Stop Minim when over:
void stop()
{
  // always close Minim audio classes when you are done with them
  song.close();
  minim.stop();
  super.stop();
}


//------------------------------
void displayText(){
  fill(255,128);
  String lineA = "GrooveFlakes";
  String lineB = "Powered by GrooveSalad";
  String lineC = "somaFM.com";
  textFont(fontA);
  
  pushMatrix();
  translate(center, center, center*1.5);
  rotateX(-PI);
  text(lineA,0,0,0);
  translate(0,20,0);
  scale(.50,.50,.50);
  text(lineB,0,0,0);
  translate(0,30,0);
  text(lineC,0,0,0);
  popMatrix();

}

void keyPressed(){
  if(key == 'p'){  
    saveFrame("GrooveFlakes-####.jpg");
    println("Screengrab!");
  }
}


